from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import mousetrap.plugins.interface as interface


class DisplayPlugin(interface.Plugin):

    def __init__(self, config):
        self._config = config
        self._window_title = config[self]['window_title']

    def run(self, app):
        app.gui.show_image(self._window_title, app.image)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

'''
Where it all begins.
'''

from argparse import ArgumentParser
from io import open
import logging
import logging.config
from os.path import dirname, expanduser, exists
import signal
import sys
import yaml

from mousetrap.config import Config
from mousetrap.core import App


class Main(object):
    DEFAULT_CONFIG_PATH = dirname(__file__) + '/mousetrap.yaml'
    USER_CONFIG_PATH = expanduser('~/.mousetrap.yaml')

    def __init__(self):
        try:
            self._app = None
            self._args = CommandLineArguments()
            self._handle_dump_annotated()
            self._config = Config().load(self._get_config_paths())
            self._handle_dump_config()
            self._configure_logging()
        except ExitException:
            sys.exit(0)

    def _get_config_paths(self):
        paths = [self.DEFAULT_CONFIG_PATH]
        if exists(self.USER_CONFIG_PATH):
            paths.append(self.USER_CONFIG_PATH)
        if self._args.config is not None:
            paths.append(self._args.config)
        return paths

    def _handle_dump_annotated(self):
        if self._args.dump_annotated:
            self._dump_annotated()
            raise ExitException()

    def _handle_dump_config(self):
        if self._args.dump_config:
            self._dump_config()
            raise ExitException()

    @classmethod
    def _dump_annotated(cls):
        with open(cls.DEFAULT_CONFIG_PATH, 'r') as annotated_file:
            print(annotated_file.read())

    def _dump_config(self):
        print(yaml.dump(dict(self._config), default_flow_style=False))

    def _configure_logging(self):
        logging.config.dictConfig(self._config['logging'])
        logger = logging.getLogger('mousetrap.main')
        logger.debug(yaml.dump(dict(self._config), default_flow_style=False))

    def run(self):
        self._app = App(self._config)
        signal.signal(signal.SIGTERM, self._stop_signal_handler)
        signal.signal(signal.SIGINT, self._stop_signal_handler)
        self._app.run()

    def _stop_signal_handler(self, signal_number, stack_frame):
        self._app.stop()


class CommandLineArguments(object):
    def __init__(self):
        parser = ArgumentParser()
        parser.add_argument(
            "--config",
            metavar="FILE",
            help="Loads configuration from FILE."
        )
        parser.add_argument(
            "--dump-config",
            help="Loads and dumps current configuration to standard out.",
            action="store_true"
        )
        parser.add_argument(
            "--dump-annotated",
            help=(
                "Dumps default configuration (with comments) to standard out."
            ),
            action="store_true"
        )
        parser.parse_args(namespace=self)


class ExitException(Exception):
    pass


def main():
    Main().run()


if __name__ == '__main__':
    main()



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division


__version__ = "3.17.3"
VERSION = __version__.split(".")



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from unittest import TestLoader
from unittest.runner import TextTestRunner
from os.path import dirname, abspath
import sys


from mousetrap.config import Config
CONFIG = Config().load_default()
print(CONFIG['camera'])


import logging
import logging.config
logging.config.dictConfig(CONFIG['logging-test'])
LOGGER = logging.getLogger('mousetrap.tests.run_python_tests')


def main():
    initialize_import_path()
    tests = load_tests()

    if not all_tests_pass(tests):
        sys.exit(1)


def initialize_import_path():
    paths = [
        get_source_directory()
    ]
    print("appending " + str(paths))
    append_to_path(paths)


def append_to_path(paths):
    for path in paths:
        sys.path.append(path)


def get_source_directory():
    return abspath(dirname(__file__) + '/../..')


def load_tests():
    directory = get_source_directory()
    tests = TestLoader().discover(directory)
    return tests


def all_tests_pass(tests):
    return TextTestRunner().run(tests).wasSuccessful()


if __name__ == '__main__':
    main()



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import gettext
import os

import logging
LOGGER = logging.getLogger(__name__)

LOCALE_DIR = os.path.abspath(
    os.path.join(os.path.dirname(os.path.realpath(__file__)), "locale")
)
LOGGER.debug("LOCALE_DIR = %s", LOCALE_DIR)

translations = gettext.translation("mousetrap", localedir=LOCALE_DIR)

try:
    _ = translations.ugettext
except AttributeError:
    _ = translations.gettext



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from yaml import safe_load
from os.path import dirname
from copy import deepcopy
from io import open


class Config(dict):

    def load(self, paths):
        for path in paths:
            self.load_path(path)
        return self

    def load_default(self):
        return self.load_path(dirname(__file__) + '/mousetrap.yaml')

    def load_path(self, path):
        print("# Loading %s" % (path))

        with open(path) as config_file:
            config = safe_load(config_file)
            _rmerge(self, config)

        return self

    def load_dict(self, dictionary):
        _rmerge(self, dictionary)

    def __getitem__(self, key):
        '''
        Allow access to class configuration by passing instance of class as
        the key. For example,

            x = config[self]['x']

        is equivelant to

            x = config['classes'][
                self.__class__.__module__ + '.' + self.__class__.__name__
            ]['x']
        '''
        from mousetrap.compat import string_types

        if isinstance(key, string_types):
            return super(Config, self).__getitem__(key)
        class_ = key.__class__
        return self['classes'][class_.__module__ + '.' + class_.__name__]


def _rmerge(target, source):
    '''
    Recursively update values in target from source.
    Only dicts are updated, all other values are deepcopied.
    '''
    if source is None:
        return
    for key, value in source.items():
        if isinstance(value, dict):
            if key not in target:
                target[key] = {}
            _rmerge(target[key], value)
        else:
            target[key] = deepcopy(value)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

'''
All things computer vision.
'''

import cv2
from mousetrap.i18n import _
from mousetrap.image import Image
import mousetrap.plugins.interface as interface

import logging
LOGGER = logging.getLogger(__name__)


FRAME_WIDTH = 3
FRAME_HEIGHT = 4


class Camera(object):
    S_CAPTURE_OPEN_ERROR = _(
        'Device #%d does not support video capture interface'
    )
    S_CAPTURE_READ_ERROR = _(
        'Error while capturing. Camera disconnected?'
    )

    def __init__(self, config):
        self._config = config
        self._device = self._new_capture_device(
            config['camera']['device_index']
        )
        self.set_dimensions(
            config['camera']['width'],
            config['camera']['height'],
        )

    @classmethod
    def _new_capture_device(cls, device_index):
        capture = cv2.VideoCapture(device_index)

        if not capture.isOpened():
            capture.release()

            raise IOError(cls.S_CAPTURE_OPEN_ERROR % device_index)

        return capture

    def set_dimensions(self, width, height):
        self._device.set(FRAME_WIDTH, width)
        self._device.set(FRAME_HEIGHT, height)

    def read_image(self):
        ret, image = self._device.read()

        if not ret:
            raise IOError(self.S_CAPTURE_READ_ERROR)

        return Image(self._config, image)


class HaarLoader(object):

    def __init__(self, config):
        self._config = config
        self._haar_files = config['haar_files']
        self._haar_cache = {}

    def from_name(self, name):
        if name not in self._haar_files:
            raise HaarNameError(name)

        haar_file = self._haar_files[name]

        haar = self.from_file(haar_file, name)

        return haar

    def from_file(self, file_, cache_name=None):
        import os

        if cache_name in self._haar_cache:
            return self._haar_cache[cache_name]

        current_dir = os.path.dirname(os.path.realpath(__file__))

        haar_file = os.path.join(current_dir, file_)

        haar = cv2.CascadeClassifier(haar_file)

        if cache_name is not None:
            if cache_name not in self._haar_cache:
                self._haar_cache[cache_name] = haar

        return haar


class HaarNameError(Exception):
    pass


class FeatureDetector(object):

    _INSTANCES = {}

    @classmethod
    def get_detector(cls, config, name, scale_factor=1.1, min_neighbors=3):
        key = (name, scale_factor, min_neighbors)

        if key in cls._INSTANCES:
            LOGGER.info("Reusing %s detector.", key)
            return cls._INSTANCES[key]

        cls._INSTANCES[key] = FeatureDetector(
            config, name, scale_factor, min_neighbors
        )

        return cls._INSTANCES[key]

    @classmethod
    def clear_all_detection_caches(cls):
        for instance in cls._INSTANCES.values():
            instance.clear_cache()

    def __init__(self, config, name, scale_factor=1.1, min_neighbors=3):
        '''
        name - name of feature to detect

        scale_factor - how much the image size is reduced at each image scale
                while searching. Default 1.1.

        min_neighbors - how many neighbors each candidate rectangle should have
                to retain it. Default 3.
        '''
        LOGGER.info(
            "Building detector: %s",
            (name, scale_factor, min_neighbors)
        )

        self._config = config
        self._name = name
        self._single = None
        self._plural = None
        self._image = None
        self._cascade = HaarLoader(config).from_name(name)
        self._scale_factor = scale_factor
        self._min_neighbors = min_neighbors
        self._last_attempt_successful = False
        self._detect_cache = {}

    def detect(self, image):
        if image in self._detect_cache:
            message = "Detection cache hit: %(image)d -> %(result)s" % {
                'image': id(image),
                'result': self._detect_cache[image],
            }
            LOGGER.debug(message)

            if isinstance(self._detect_cache[image], FeatureNotFoundException):
                message = str(self._detect_cache[image])
                raise FeatureNotFoundException(
                    message,
                    cause=self._detect_cache[image]
                )

            return self._detect_cache[image]
        try:
            self._image = image
            self._detect_plural()
            self._exit_if_none_detected()
            self._unpack_first()
            self._extract_image()
            self._calculate_center()
            self._detect_cache[image] = self._single

            return self._detect_cache[image]
        except FeatureNotFoundException as exception:
            self._detect_cache[image] = exception

            raise

    def _detect_plural(self):
        self._plural = self._cascade.detectMultiScale(
            self._image.to_cv_grayscale(),
            self._scale_factor,
            self._min_neighbors,
        )

    def _exit_if_none_detected(self):
        if len(self._plural) == 0:
            message = _('Feature not detected: %s') % (self._name)
            if self._last_attempt_successful:
                self._last_attempt_successful = False
                LOGGER.info(message)
            raise FeatureNotFoundException(message)
        else:
            if not self._last_attempt_successful:
                self._last_attempt_successful = True
                message = _('Feature detected: %s') % (self._name)
                LOGGER.info(message)

    def _unpack_first(self):
        self._single = dict(
            zip(['x', 'y', 'width', 'height'], self._plural[0])
        )

    def _calculate_center(self):
        self._single["center"] = {
            "x": (self._single["x"] + self._single["width"]) // 2,
            "y": (self._single["y"] + self._single["height"]) // 2,
        }

    def _extract_image(self):
        single = self._single
        from_y = single['y']
        to_y = single['y'] + single['height']
        from_x = single['x']
        to_x = single['x'] + single['width']
        image_cv_grayscale = self._image.to_cv_grayscale()
        single["image"] = Image(
            self._config,
            image_cv_grayscale[from_y:to_y, from_x:to_x],
            is_grayscale=True,
        )

    def clear_cache(self):
        self._detect_cache.clear()


class FeatureDetectorClearCachePlugin(interface.Plugin):
    def __init__(self, config):
        super(FeatureDetectorClearCachePlugin, self).__init__(config)
        self._config = config

    def run(self, app):
        FeatureDetector.clear_all_detection_caches()


class FeatureNotFoundException(Exception):
    def __init__(self, message, cause=None):
        if cause is not None:
            message = message + ', caused by ' + repr(cause)
        self.cause = cause
        super(FeatureNotFoundException, self).__init__(message)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import unittest


from .patches import GtkGdkPatch


class test_Observable(unittest.TestCase):

    def setUp(self):
        self.gdk_patcher = GtkGdkPatch()
        self.gdk_patcher.patch_in_setup(test_case=self)

        from mousetrap.core import Observable
        self.observable = Observable()
        self.client1 = Client()
        self.client2 = Client()

    def test_observable_callback_noArguments_success(self):
        self.observable.subscribe(self.client1)
        self.observable._fire('callback')
        self.assertTrue(
            len(self.client1.callback_params) == 1,
            msg="callback not called."
        )

    def test_observable_callback_withArguments_success(self):
        self.observable._add_argument('param', 'param')
        self.observable.subscribe(self.client1)
        self.observable._fire('callback')
        self.assertEquals(
            'param',
            self.client1.callback_params[0]['param'],
            msg="param not passed correctly."
        )

    def test_multiple_subscribers(self):
        self.observable.subscribe(self.client1)
        self.observable.subscribe(self.client2)
        self.observable._fire('callback')
        self.assertEqual(
            len(self.client1.callback_params),
            1,
            msg="callback not called on client1."
        )
        self.assertEqual(
            len(self.client2.callback_params),
            1,
            msg="callback not called on client2."
        )


class Client(object):

    def __init__(self):
        self.callback_params = []

    def callback(self, param=None):
        self.callback_params.append({
            'param': param,
        })


class test_Loop(unittest.TestCase):

    def setUp(self):
        from mousetrap.core import Loop

        self.gdk_patcher = GtkGdkPatch()
        self.gdk_patcher.patch_in_setup(test_case=self)

        self.config = {'loops_per_second': 10}
        self.loop = Loop(self.config, app=None)

    def test_loop(self):
        self.loop.start()


if __name__ == '__main__':
    unittest.main()



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import unittest
from mousetrap.main import Config


from .patches import GtkGdkPatch


class test_pointer(unittest.TestCase):

    def setUp(self):
        self.gtk_gdk_patcher = GtkGdkPatch()
        self.gtk_gdk_patcher.patch_in_setup(test_case=self)

        from mousetrap.gui import Pointer
        self.pointer = Pointer(Config().load_default())

    def test_get_position(self):
        pointer_x, pointer_y = self.pointer.get_position()

        try:
            pointer_x += 1
            pointer_y += 1
        except TypeError:
            self.assertTrue(
                False,
                msg='pointer_x or pointer_y is not a number'
            )

    def test_set_position(self):
        try:
            # Python 3
            import unittest.mock as mock
        except ImportError:
            # Python 2
            import mock

        self.pointer.get_position = mock.MagicMock(return_value=(3, 4))
        self.pointer.set_position((3, 4))
        pointer_x, pointer_y = self.pointer.get_position()
        self.assertEquals(3, pointer_x)
        self.assertEquals(4, pointer_y)


if __name__ == '__main__':
    unittest.main()



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from mousetrap.i18n import _


class Plugin(object):
    def __init__(self, config):
        '''Override to initialize and configure yourself.
        (Do not call parent/this constructor.)'''
        pass

    def run(self, app):
        '''Called each pass of the loop.'''
        raise NotImplementedError(_('Must implement.'))



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

import mousetrap.plugins.interface as interface
from mousetrap.vision import FeatureDetector, FeatureNotFoundException
from mousetrap.gui import Gui


class NosePlugin(interface.Plugin):
    def __init__(self, config):
        self._config = config
        self._nose_locator = NoseLocator()
        self._gui = Gui()
        self._location = None
        self._image = None

    def update_image(self, image):
        self._image = image
        try:
            point_image = self._nose_locator.locate(image)
            point_screen = self._convert_image_to_screen_point(*point_image)
            self._location = point_screen
        except FeatureNotFoundException:
            self._location = None

    def _convert_image_to_screen_point(self, image_x, image_y):
        image_width = self._image.get_width()
        image_height = self._image.get_height()
        percent_x = 1.0 * image_x // image_width
        percent_y = 1.0 * image_y // image_height
        screen_x = percent_x * self._gui.get_screen_width()
        screen_y = percent_y * self._gui.get_screen_height()
        half_width = self._gui.get_screen_width() // 2
        screen_x = (-1 * (screen_x - half_width)) + half_width
        return (screen_x, screen_y)

    def get_new_position(self):
        return self._location


class NoseJoystickPlugin(interface.Plugin):

    def __init__(self, config):
        self._config = config
        self._threshold = config[self]['threshold']
        self._nose_locator = NoseLocator(config)
        self._initial_image_location = (0, 0)
        self._last_delta = (0, 0)

    def run(self, app):
        self._app = app
        location = None
        try:
            point_image = self._nose_locator.locate(app.image)
            point_screen = self._convert_image_to_screen_point(*point_image)
            location = point_screen
        except FeatureNotFoundException:
            location = app.pointer.get_position()
            location = self._apply_delta_to_point(location, self._last_delta)
        app.pointer.set_position(location)

    def _apply_delta_to_point(self, point, delta):
        delta_x, delta_y = delta
        point_x, point_y = point

        if delta_x == 0 and delta_y == 0:
            return None

        point_x += delta_x
        point_y += delta_y

        return (point_x, point_y)

    def _convert_image_to_screen_point(self, image_x, image_y):
        initial_x, initial_y = self._initial_image_location

        if initial_x == 0 and initial_y == 0:
            self._initial_image_location = (image_x, image_y)

            return self._initial_image_location

        delta_x = initial_x - image_x
        delta_y = image_y - initial_y

        if abs(delta_x) < self._threshold:
            delta_x = 0

        if abs(delta_y) < self._threshold:
            delta_y = 0

        delta = (delta_x, delta_y)

        self._last_delta = delta

        location = self._app.pointer.get_position()

        return self._apply_delta_to_point(location, delta)


class NoseLocator(object):
    def __init__(self, config):
        self._config = config
        self._face_detector = FeatureDetector.get_detector(
            config,
            'face',
            scale_factor=config[self]['face_detector']['scale_factor'],
            min_neighbors=config[self]['face_detector']['min_neighbors'],
        )
        self._nose_detector = FeatureDetector.get_detector(
            config,
            'nose',
            scale_factor=config[self]['nose_detector']['scale_factor'],
            min_neighbors=config[self]['nose_detector']['min_neighbors'],
        )

    def locate(self, image):
        face = self._face_detector.detect(image)
        nose = self._nose_detector.detect(face['image'])
        return (
            face['x'] + nose['center']['x'],
            face['y'] + nose['center']['y'],
        )



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
'''
All things GUI.
'''


import logging
LOGGER = logging.getLogger(__name__)


# gtk can be patch with a Mock during testing without ever importing Gtk
# from gi.repository. This is necessary since importing Gtk from gi.repository
# on a headless system raises an error.
gtk = None


def get_gtk():
    global gtk

    if gtk is None:
        from gi.repository import Gtk
        gtk = Gtk

    return gtk

# gdk can be patch with a Mock during testing without ever importing Gdk
# from gi.repository. This is necessary since importing Gdk from gi.repository
# on a headless system raises an error.
gdk = None


def get_gdk():
    global gdk

    if gdk is None:
        from gi.repository import Gdk
        gdk = Gdk

    return gdk


from Xlib.display import Display as XlibDisplay
from Xlib.ext import xtest
from Xlib import X


from mousetrap.i18n import _


class ImageWindow(object):

    def __init__(self, config, message):
        self._config = config
        self._window = get_gtk().Window(title=message)
        self._canvas = get_gtk().Image()
        self._window.add(self._canvas)
        self._window.connect("delete-event", get_gtk().main_quit)
        self._window.show_all()

    def draw(self, image):
        '''Draw image to this window.
        '''
        image = image.to_pixbuf()
        self._canvas.set_from_pixbuf(image)
        self._canvas.queue_draw()


class Gui(object):
    _running = False

    @classmethod
    def start(cls):
        '''Start handling events.'''
        if not cls._running:
            cls._running = True
            get_gtk().main()

    @classmethod
    def stop(cls):
        '''Stop handling events.'''
        if cls._running:
            cls._running = False
            get_gtk().main_quit()

    def __init__(self, config):
        self._config = config
        self._windows = {}

    def show_image(self, window_name, image):
        '''Displays image in window named by window_name.
           May reuse named windows.
           '''
        if window_name not in self._windows:
            self._windows[window_name] = ImageWindow(self._config, window_name)
        self._windows[window_name].draw(image)

    def get_screen_width(self):
        return get_gtk().Window().get_screen().get_width()

    def get_screen_height(self):
        return get_gtk().Window().get_screen().get_height()


class Pointer(object):
    BUTTON_LEFT = X.Button1

    def __init__(self, config):
        self._config = config
        gdk_display = get_gdk().Display.get_default()
        device_manager = gdk_display.get_device_manager()
        self._pointer = device_manager.get_client_pointer()
        self._screen = gdk_display.get_default_screen()
        self._moved = False

    def set_position(self, position=None):
        '''Move pointer to position (x, y). If position is None,
        no change is made.'''
        self._moved = False
        if position is not None:
            LOGGER.debug(_('Moving pointer to %s'), position)

            self._pointer.warp(self._screen, position[0], position[1])
            self._moved = True
        else:
            LOGGER.debug(_('Not moving the pointer'))

    def is_moving(self):
        '''Returns True if last call to set_position passed a non-None value
        for position.'''
        return self._moved

    def get_position(self):
        x_index = 1
        y_index = 2
        position = self._pointer.get_position()
        return (position[x_index], position[y_index])

    def click(self, button=BUTTON_LEFT):
        display = XlibDisplay()
        for event, button in \
                [(X.ButtonPress, button), (X.ButtonRelease, button)]:
            LOGGER.debug('%s %s', event, button)
            xtest.fake_input(display, event, button)
            display.sync()



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import logging
LOGGER = logging.getLogger(__name__)

from gi.repository import GLib

from mousetrap.i18n import _
from mousetrap.gui import Gui, Pointer
from mousetrap.vision import Camera


class App(object):

    def __init__(self, config):
        LOGGER.info("Initializing")
        self.config = config
        self.image = None
        self.loop = Loop(config, self)
        self.gui = Gui(config)
        self.camera = Camera(config)
        self.pointer = Pointer(config)
        self.plugins = []
        self._assemble_plugins()

    def _assemble_plugins(self):
        self._load_plugins()
        self._register_plugins_with_loop()

    def _load_plugins(self):
        for class_ in self.config['assembly']:
            self.plugins.append(self._load_plugin(class_))

    def _load_plugin(self, class_string):
        try:
            LOGGER.info('loading %s', class_string)

            class_path = class_string.split('.')
            module = __import__(
                '.'.join(class_path[:-1]), {}, {}, class_path[-1]
            )

            return getattr(module, class_path[-1])(self.config)
        except ImportError:
            LOGGER.error(
                _(
                    'Could not import plugin `%s`. ' +
                    'Check config file and PYTHONPATH.'
                ),
                class_string
            )

            raise

    def _register_plugins_with_loop(self):
        for plugin in self.plugins:
            self.loop.subscribe(plugin)

    def run(self):
        self.loop.start()
        self.gui.start()

    def stop(self):
        self.gui.stop()
        self.loop.stop()


class Observable(object):

    def __init__(self):
        self.__observers = []
        self.__arguments = {}

    def subscribe(self, observer):
        self.__observers.append(observer)

    def _add_argument(self, key, value):
        self.__arguments[key] = value

    def _fire(self, callback_name):
        for observer in self.__observers:
            callback = getattr(observer, callback_name)
            callback(**self.__arguments)


class Loop(Observable):
    MILLISECONDS_PER_SECOND = 1000.0
    CALLBACK_RUN = 'run'

    def __init__(self, config, app):
        super(Loop, self).__init__()
        self._config = config
        self._interval = None
        self._loops_per_second = None
        self._timeout_id = None
        self._set_loops_per_second(config['loops_per_second'])
        self._add_argument('app', app)
        self._loop_enabled = False

    def _set_loops_per_second(self, loops_per_second):
        self._loops_per_second = loops_per_second
        self._interval = int(round(
            self.MILLISECONDS_PER_SECOND / self._loops_per_second))

    def start(self):
        self._loop_enabled = True
        self._timeout_id = GLib.timeout_add(self._interval, self._run)

    def stop(self):
        self._loop_enabled = False

    def _run(self):
        self._fire(self.CALLBACK_RUN)
        return self._loop_enabled



from setuptools import setup, find_packages
from distutils.command.build import build
from setuptools.command.egg_info import egg_info
import glob
import os
import sys


SRC_PATH = os.path.relpath(os.path.join(os.path.dirname(__file__), "src"))
ABS_SRC_PATH = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), "src"))

sys.path.append(SRC_PATH)

from mousetrap import __version__

PYTHON_VERSION = sys.version_info

LOCALE_PATH = "src/mousetrap/locale"

all_requirements = [
    "pygobject",
    "pyyaml",
]

python2_requirements = all_requirements + [
    "python-xlib",
]

python3_requirements = all_requirements + [
    "python3-xlib",
]

if PYTHON_VERSION[0] > 2:
    requirements = python3_requirements
else:
    requirements = python2_requirements


class EggInfoCommand(egg_info):

    def run(self):
        try:
            import gi.repository
        except ImportError:
            sys.stderr.write(
"""
PyGObject does not appear to be installed.  This cannot be installed
automatically and must be installed to the system using your package manager.

On apt-based systems:

    sudo apt-get install python-gobject

On yum-based systems:

    sudo yum install python-gobject

The installation not work as expected without this dependency.
""")

        try:
            import Xlib
        except ImportError:
            # Python3-xlib can be installed automatically
            if PYTHON_VERSION[0] < 3:
                sys.stderr.write(
    """
    Python Xlib does not appear to be installed.  This cannot be installed
    automatically and must be installed to the system using your package manager.

    On apt-based systems:

        sudo apt-get install python-xlib

    On yum-based systems:

        sudo yum install python-xlib

    The installation not work as expected without this dependency.
    """)

        try:
            import cv2
        except ImportError:
            sys.stderr.write(
"""
OpenCV does not appear to be installed.  This cannot be installed
automatically and must be installed to the system using your package manager.

On apt-based systems:

    sudo apt-get install python-opencv

On yum-based systems:

    sudo yum install python-opencv

The installation not work as expected without this dependency.
""")

        if "build" in self.distribution.command_obj:
            build_command = self.distribution.command_obj["build"]

            self.egg_base = build_command.build_base

            self.egg_info = os.path.join(self.egg_base, os.path.basename(self.egg_info))

        egg_info.run(self)


class BuildCommand(build):

    def run(self):
        from subprocess import Popen

        sys.stdout.write("Compiling locale files...\n")

        program = "msgfmt"

        LOCALE_PATH = "%s/mousetrap/locale" % SRC_PATH
        DEST_PATH = "%s/mousetrap/locale" % self.build_lib

        root, directories, files = next(os.walk(LOCALE_PATH))

        language_codes = directories

        for language_code in language_codes:
            message_file = os.path.join(LOCALE_PATH, language_code, "LC_MESSAGES", "mousetrap.po")
            compiled_file = os.path.join(DEST_PATH, language_code, "LC_MESSAGES", "mousetrap.mo")

            if not os.path.exists(os.path.dirname(compiled_file)):
                sys.stdout.write("Creating %s\n" % os.path.dirname(compiled_file))
                os.makedirs(os.path.dirname(compiled_file))

            arguments = [message_file, "--output-file", compiled_file]

            sys.stdout.write("Compiling %s locale" % language_code)

            command = [program] + arguments

            process = Popen(command)

            output, errors = process.communicate()
            status_code = process.returncode

            if status_code == 0:
                sys.stdout.write(" [OK]\n")
            else:
                sys.stdout.write(" [FAIL]\n")

        build.run(self)


setup(
    name="mousetrap",
    version=__version__,
    url="https://wiki.gnome.org/Projects/MouseTrap",
    license="GPL",
    include_package_data=True,
    install_requires=requirements,
    packages=find_packages(SRC_PATH),
    cmdclass={
        "build": BuildCommand,
        "egg_info": EggInfoCommand,
    },
    package_dir={
        "": SRC_PATH,
    },
    entry_points={
        "console_scripts": [
            "mousetrap = mousetrap.main:main",
        ],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
    ],
)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import unittest
from mousetrap.config import _rmerge, Config
from io import open


class test__rmerge(unittest.TestCase):

    def setUp(self):
        self.a = {
            'red': 1,
            'white': ['washington', [2, 3], {'lincoln': 4}],
            'blue': {
                'alpha': 5,
                'list': [6, 7],
                'dict': {
                    'charlie': 8}}}

        self.b = {
            'new': 9,
            'white': ['replacement'],
            'blue': {
                'new': 10,
                'dict': {
                    'charlie': 11,
                    'new': 12,
                    'newdict': {
                        'some': 'dict'}}}}
        self.ab = {
            'new': 9,
            'red': 1,
            'white': ['replacement'],
            'blue': {
                'new': 10,
                'alpha': 5,
                'list': [6, 7],
                'dict': {
                    'charlie': 11,
                    'new': 12,
                    'newdict': {
                        'some': 'dict'}}}}

    def test__rmerge(self):
        _rmerge(self.a, self.b)
        self.assertEqual(self.ab, self.a)

    def test__rmerge_None(self):
        from copy import deepcopy
        original = deepcopy(self.a)
        _rmerge(self.a, None)
        self.assertEqual(original, self.a)


class test_Config(unittest.TestCase):
    def setUp(self):
        self.config = Config()
        self.files = Files()

    def tearDown(self):
        self.files.delete()

    def test_load_default(self):
        self.config.load_default()
        self.assertIsInstance(self.config['assembly'], list)

    def test_load(self):
        self.files.write('f1', (
            "x: 1\n"
            "y: 2"
        ))
        self.files.write('f2', (
            "x: 3\n"
            "z: 4"
        ))

        self.config.load([self.files.path('f1'), self.files.path('f2')])
        self.assertEquals({'x': 3, 'y': 2, 'z': 4}, self.config)

    def test_classes_config(self):
        self.config.load_dict({
            'classes': {
                self.__class__.__module__ + '.' + self.__class__.__name__: {
                    'x': 4}}})
        self.assertEquals(4, self.config[self]['x'])


class Files(object):
    def __init__(self, directory=None):
        if directory is None:
            directory = __file__ + '.data'
        from os.path import exists
        from os import mkdir
        if not exists(directory):
            mkdir(directory)
        self.directory = directory

    def write(self, file_name, data):
        with open(self.path(file_name), 'w') as data_file:
            data_file.write(data)

    def read(self, file_name, data):
        with open(self.path(file_name), 'r') as data_file:
            return data_file.read()

    def path(self, file_name):
        return self.directory + '/' + file_name

    def delete(self):
        from shutil import rmtree
        rmtree(self.directory)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
'''
All things image manipulation.
'''

import cv2
from gi.repository import GdkPixbuf

_GDK_PIXBUF_BIT_PER_SAMPLE = 8


class Image(object):
    def __init__(self, config, image_cv, is_grayscale=False):
        self._config = config
        self._image_cv = image_cv
        self._is_grayscale = is_grayscale
        self._image_cv_grayscale = None
        if self._is_grayscale:
            self._image_cv_grayscale = self._image_cv

    def to_cv(self):
        return self._image_cv

    def to_cv_grayscale(self):
        if self._image_cv_grayscale is None:
            self._image_cv_grayscale = _cv_rgb_to_cv_grayscale(self._image_cv)
        return self._image_cv_grayscale

    def to_pixbuf(self):
        return _cvimage_to_pixbuf(self._image_cv)

    def get_width(self):
        return self._image_cv.shape[0]

    def get_height(self):
        return self._image_cv.shape[1]


def _cv_rgb_to_cv_grayscale(image):
    return cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)


def _cvimage_to_pixbuf(cvimage):
    # Convert cvimage's colorspace from RGB to BGR for correct color display
    cvimage = cv2.cvtColor(cvimage, cv2.COLOR_RGB2BGR)

    data = cvimage.tostring()
    colorspace = GdkPixbuf.Colorspace.RGB
    has_alpha_channel = False
    width = cvimage.shape[1]
    height = cvimage.shape[0]

    # dist in bytes between row starts
    row_stride = cvimage.strides[0]

    # Function used to free the data when the pixbuf's reference count drops to
    # zero, or None if the data should not be freed.
    destroy_fn = None

    # Closure data to pass to the destroy notification function.
    destroy_fn_data = None

    return GdkPixbuf.Pixbuf.new_from_data(
        data,
        colorspace,
        has_alpha_channel,
        _GDK_PIXBUF_BIT_PER_SAMPLE,
        width,
        height,
        row_stride,
        destroy_fn,
        destroy_fn_data,
    )



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division


try:
    # Python 3
    import unittest.mock as mock
except ImportError:
    # Python 2
    import mock


class GtkGdkPatch:
    def __init__(self):
        self.patchers = {}
        self.mocks = {}

    def patch_in_setup(self, test_case):
        self._patch('gtk', test_case)
        self._patch('gdk', test_case)

    def _patch(self, thing, test_case):
        self.patchers[thing] = mock.patch('mousetrap.gui.' + thing)
        self.mocks[thing] = self.patchers[thing].start()
        test_case.addCleanup(self.patchers[thing].stop)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
import unittest
from mousetrap.vision import Camera
from mousetrap.main import Config


class test_camera(unittest.TestCase):

    def setUp(self):
        self.camera = Camera(Config().load_default())


if __name__ == '__main__':
    unittest.main()



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

from mousetrap.i18n import _
import logging
LOGGER = logging.getLogger(__name__)


import mousetrap.plugins.interface as interface
from mousetrap.vision import FeatureDetector, FeatureNotFoundException


class EyesPlugin(interface.Plugin):
    def __init__(self, config):
        self._config = config
        self._motion_detector = MotionDetector(config)
        self._closed_detector = ClosedDetector(config)

    def run(self, app):
        self._motion_detector.update(app.pointer)
        self._closed_detector.update(app.image)

        if self._motion_detector.is_stationary() and \
                self._closed_detector.is_closed():
            self._closed_detector.reset()
            app.pointer.click()


class MotionDetector(object):
    def __init__(self, config):
        self._config = config
        self._max_samples = config[self]['max_samples']
        self._history = History(config, self._max_samples)

    def update(self, pointer):
        self._history.append(pointer.get_position())

    def is_stationary(self):
        last_point = self._history[-1]
        return all([point == last_point for point in self._history])


class ClosedDetector(object):
    def __init__(self, config):
        self._config = config
        self._max_samples = config[self]['max_samples']
        self._min_fraction_to_be_closed = config[self][
            'min_fraction_to_be_closed'
        ]
        self._min_misses_to_be_closed = int(
            self._min_fraction_to_be_closed * self._max_samples)
        self._left_locator = LeftEyeLocator(config)
        self._detection_history = History(config, self._max_samples)

    def update(self, image):
        self._detection_history.append(self._left_locator.locate(image))

    def is_closed(self):
        misses = self._detection_history.count(False)
        return misses > self._min_misses_to_be_closed

    def reset(self):
        self._detection_history.clear()


class LeftEyeLocator(object):

    def __init__(self, config):
        self._config = config
        self._face_detector = FeatureDetector.get_detector(
            config,
            "face",
            scale_factor=config[self]['face_detector']['scale_factor'],
            min_neighbors=config[self]['face_detector']['min_neighbors'],
        )
        self._open_eye_detector = FeatureDetector.get_detector(
            config,
            "open_eye",
            scale_factor=config[self]['open_eye_detector']['scale_factor'],
            min_neighbors=config[self]['open_eye_detector']['min_neighbors'],
        )
        self._left_eye_detector = FeatureDetector.get_detector(
            config,
            "left_eye",
            scale_factor=config[self]['left_eye_detector']['scale_factor'],
            min_neighbors=config[self]['left_eye_detector']['min_neighbors'],
        )

    def locate(self, image):
        face = None

        try:
            face = self._face_detector.detect(image)

            LOGGER.debug(_("Found the face"))
        except FeatureNotFoundException:
            return True

        try:
            left_eye = self._left_eye_detector.detect(face["image"])

            LOGGER.debug(_("Found the left eye at %s"), left_eye)
        except FeatureNotFoundException:
            return True

        try:
            open_eye = self._open_eye_detector.detect(face["image"])

            LOGGER.debug(_("Found an open eye at %s"), open_eye)

            return True
        except FeatureNotFoundException:
            return False


class History(list):
    def __init__(self, config, max_length):
        super(History, self).__init__()
        self._config = config
        self._max_length = max_length

    def append(self, value):
        super(History, self).append(value)
        while len(self) > self._max_length:
            del self[0]

    def clear(self):
        del self[:]
